#include <hlog.hxx>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <syslog.h>

static log_base *LOG=NULL;

log_base::log_base()
{
  prev=LOG;
  LOG=this;
  if (prev==NULL) { _debugLevel=0; }
  else { _debugLevel=prev->_debugLevel; }
}

log_base::~log_base()
{
  LOG=prev;
}

void log_base::debugLevel(int level)
{
  _debugLevel=level;
}

//*************************************************************************

void log_cerr::log(int ,char *format,va_list arg)
{
  vfprintf(stderr,format,arg);
  fprintf(stderr,"\n");
}

void log_cerr::__debug(int type,char *format,va_list arg)
{
  if (_debugLevel!=0) {
    fprintf(stderr,"debugging (type=%d): ",type);
    vfprintf(stderr,format,arg);
    fprintf(stderr,"\n");
  }
}

//*************************************************************************

log_syslog::log_syslog(char *prgname)
{
  openlog(prgname,LOG_PID,LOG_USER);
}

log_syslog::~log_syslog()
{
  closelog();
}

void log_syslog::log(int ,char *format,va_list arg)
{
char msg[4096];
  vsprintf(msg,format,arg);
  syslog(LOG_INFO,"%s",msg);
}

void log_syslog::__debug(int type, char *format,va_list arg)
{
  if (_debugLevel!=0) {char msg[4096],msg1[4096];
    vsprintf(msg,format,arg);
    sprintf(msg1,"debugging (type=%d): %s",type,msg);
    syslog(LOG_INFO,"%s",msg1);
  }
}

//*************************************************************************

void log(int type,char *format,...)
{
va_list arg;
  va_start(arg,format);
  if (LOG==NULL) {
    log_cerr l;
      l.log(type,format,arg);
  }
  else {
    LOG->log(type,format,arg);
  }
  va_end(arg);
}

void __debug(int type,char *format,...)
{
va_list arg;
  va_start(arg,format);
  if (LOG==NULL) {
    log_cerr l;
      l.__debug(type,format,arg);
  }
  else {
    LOG->__debug(type,format,arg);
  }
  va_end(arg);
}

bool debugOn(void) 
{
  if (LOG==NULL) {
    return false;
  }
  else {
    return LOG->debugOn();
  }
}


